
# CPL_CHECK_FFTW
#------------------
# Checks for the cfitsio library and header files.
AC_DEFUN([CPL_CHECK_FFTW],
[
    AC_MSG_CHECKING([for fftw])

    fftw_check_header="fitsio.h"
    fftw_check_lib="libfftw.a"

    fftw_includes=""
    fftw_libraries=""

    AC_ARG_WITH(fftw,
                AC_HELP_STRING([--with-fftw],
                               [location where fftw is installed]),
                [
                    cpl_with_fftw_includes=$withval/include
                    cpl_with_fftw_libs=$withval/lib
                ])

    AC_ARG_WITH(fftw-includes,
                AC_HELP_STRING([--with-fftw-includes],
                               [location of the fftw header files]),
                cpl_with_fftw_includes=$withval)

    AC_ARG_WITH(fftw-libs,
                AC_HELP_STRING([--with-fftw-libs],
                               [location of the fftw library]),
                cpl_with_fftw_libs=$withval)

    AC_ARG_ENABLE(fftw-test,
                  AC_HELP_STRING([--disable-fftw-test],
                                 [disables checks for the fftw library and headers]),
                  cpl_enable_fftw_test=$enableval,
                  cpl_enable_fftw_test=yes)


    if test "x$cpl_enable_fftw_test" = xyes; then

        # Check for the fftw includes

        if test -z "$cpl_with_fftw_includes"; then
            fftw_incdirs="/opt/fftw/include /usr/local/include /usr/include"

            test -n "$CPLDIR" && fftw_incdirs="$CPLDIR/include \
                $fftw_incdirs"

            test -n "$FFTWDIR" && fftw_incdirs="$FFTWDIR/include \
                $fftw_incdirs"
        else
            fftw_incdirs="$cpl_with_fftw_includes"
        fi

        ESO_FIND_FILE($fftw_check_header, $fftw_incdirs,
                      fftw_includes)


        # Check for the fftw library

        if test -z "$cpl_with_fftw_libs"; then
            fftw_libdirs="/opt/fftw/lib /usr/local/lib /usr/lib"

            test -n "$CPLDIR" && fftw_libdirs="$CPLDIR/lib \
                $fftw_libdirs"

            test -n "$FFTWDIR" && fftw_libdirs="$FFTWDIR/lib \
                $fftw_libdirs"
        else
            fftw_libdirs="$cpl_with_fftw_libs"
        fi

        ESO_FIND_FILE($fftw_check_lib, $fftw_libdirs,
                      fftw_libraries)


        if test x"$fftw_includes" = xno || \
            test x"$fftw_libraries" = xno; then
            fftw_notfound=""

            if test x"$fftw_includes" = xno; then
                if test x"$fftw_libraries" = xno; then
                    fftw_notfound="(headers and libraries)"
                else
                    fftw_notfound="(headers)"
                fi
            else
                fftw_notfound="(libraries)"
            fi

            AC_MSG_ERROR([fftw $fftw_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $fftw_libraries, headers $fftw_includes])
        fi

        # Set up the symbols

        FFTW_INCLUDES="-I$fftw_includes"
        FFTW_LDFLAGS="-L$fftw_libraries"
        LIBFFTW="-lfftw"
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([fftw checks have been disabled! This package may not build!])
        FFTW_INCLUDES=""
        FFTW_LDFLAGS=""
        LIBFFTW=""
    fi

    AC_SUBST(FFTW_INCLUDES)
    AC_SUBST(FFTW_LDFLAGS)
    AC_SUBST(LIBFFTW)

])



# CPL_CHECK_QFITS
#----------------
# Checks for the Qfits library and header files.
AC_DEFUN([CPL_CHECK_QFITS],
[

    AC_MSG_CHECKING([for Qfits])

    cpl_qfits_check_header="qfits.h"
    cpl_qfits_check_lib="libqfits.a"

    cpl_qfits_includes=""
    cpl_qfits_libraries=""

    AC_ARG_WITH(qfits,
                AC_HELP_STRING([--with-qfits],
                               [location where Qfits is installed]),
                [
                    cpl_with_qfits_includes=$withval/include
                    cpl_with_qfits_libs=$withval/lib
                ])

    AC_ARG_WITH(qfits-includes,
                AC_HELP_STRING([--with-qfits-includes],
                               [location of the Qfits header files]),
                cpl_with_qfits_includes=$withval)

    AC_ARG_WITH(qfits-libs,
                AC_HELP_STRING([--with-qfits-libs],
                               [location of the Qfits library]),
                cpl_with_qfits_libs=$withval)

    AC_ARG_ENABLE(qfits-test,
                  AC_HELP_STRING([--disable-qfits-test],
                                 [disables checks for the Qfits library and headers]),
                  cpl_enable_qfits_test=$enableval,
                  cpl_enable_qfits_test=yes)


    if test "x$cpl_enable_qfits_test" = xyes; then

        # Check for the Qfits includes

        if test -z "$cpl_with_qfits_includes"; then
            cpl_qfits_incdirs="/opt/qfits/include /usr/local/include /usr/include"

            test -n "$CPLDIR" && cpl_qfits_incdirs="$CPLDIR/include \
                                                    $cpl_qfits_incdirs"

            test -n "$QFITSDIR" && cpl_qfits_incdirs="$QFITSDIR/include \
                                                      $cpl_qfits_incdirs"
        else
            cpl_qfits_incdirs="$cpl_with_qfits_includes"
        fi

        ESO_FIND_FILE($cpl_qfits_check_header, $cpl_qfits_incdirs,
                      cpl_qfits_includes)


        # Check for the Qfits library

        if test -z "$cpl_with_qfits_libs"; then
            cpl_qfits_libdirs="/opt/qfits/lib /usr/local/lib /usr/lib"

            test -n "$CPLDIR" && cpl_qfits_libdirs="$CPLDIR/lib \
                                                    $cpl_qfits_libdirs"

            test -n "$QFITSDIR" && cpl_qfits_libdirs="$QFITSDIR/lib \
                                                      $cpl_qfits_libdirs"
        else
            cpl_qfits_libdirs="$cpl_with_qfits_libs"
        fi

        ESO_FIND_FILE($cpl_qfits_check_lib, $cpl_qfits_libdirs,
                      cpl_qfits_libraries)


        if test x"$cpl_qfits_includes" = xno || \
            test x"$cpl_qfits_libraries" = xno; then
            cpl_qfits_notfound=""

            if test x"$cpl_qfits_includes" = xno; then
                if test x"$cpl_qfits_libraries" = xno; then
                    cpl_qfits_notfound="(headers and libraries)"
                else            
                    cpl_qfits_notfound="(headers)"
                fi
            else
                cpl_qfits_notfound="(libraries)"
            fi

            AC_MSG_ERROR([Qfits $cpl_qfits_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_qfits_libraries, headers $cpl_qfits_includes])
        fi


        # Set up the symbols

        QFITS_INCLUDES="-I$cpl_qfits_includes"
        QFITS_LDFLAGS="-L$cpl_qfits_libraries"
        LIBQFITS="-lqfits"
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([Qfits checks have been disabled! This package may not build!])
        QFITS_INCLUDES=""
        QFITS_LDFLAGS=""
        LIBQFITS=""
    fi

    AC_SUBST(QFITS_INCLUDES)
    AC_SUBST(QFITS_LDFLAGS)
    AC_SUBST(LIBQFITS)

    # Check whether we are using Qfits 4.3.5 or a later version

    AC_CACHE_CHECK([whether Qfits provides qfits_get_datetime_iso8601],
                   [cpl_cv_func_qfits_get_datetime_iso8601],
                   [
                       AC_LANG_PUSH(C)

                       cpl_cppflags_save="$CPPFLAGS"
                       cpl_cflags_save="$CFLAGS"
                       cpl_ldflags_save="$LDFLAGS"
                       cpl_libs_save="$LIBS"

                       if test x$GCC = xyes; then
                           CFLAGS="$CFLAGS -pedantic-errors"
                           CPPFLAGS="$CPPFLAGS $CFLAGS"
                       fi

                       CFLAGS="$CFLAGS $QFITS_INCLUDES"
                       LDFLAGS="$LDFLAGS $QFITS_LDFLAGS"
                       LIBS="$LIBS $LIBQFITS"

                       AC_RUN_IFELSE([
#include <qfits.h>

int
main(void)
{

    const char *s = NULL;

    s = qfits_get_datetime_iso8601();
    return 0;

}
                                     ],
                                     cpl_cv_func_qfits_get_datetime_iso8601=yes,
                                     cpl_cv_func_qfits_get_datetime_iso8601=no,
                                     cpl_cv_func_qfits_get_datetime_iso8601=no)


                       CPPFLAGS="$cpl_cppflags_save"
                       CFLAGS="$cpl_cflags_save"
                       LDFLAGS="$cpl_ldflags_save"
                       LIBS="$cpl_libs_save"

                       AC_LANG_POP(C)

                   ])

    AH_TEMPLATE([HAVE_QFITS_GET_DATETIME_ISO8601],
                [Define if Qfits provides the `qfits_get_datetime_iso8601' 
                 function])

    if test x$cpl_cv_func_qfits_get_datetime_iso8601 = xyes; then
        AC_DEFINE(HAVE_QFITS_GET_DATETIME_ISO8601)
    fi

])


# CPL_CHECK_CEXT
#---------------
# Checks for the C extension library and header files.
AC_DEFUN([CPL_CHECK_CEXT],
[

    AC_MSG_CHECKING([for libcext])

    cpl_cext_check_header="cxmacros.h"
    cpl_cext_check_lib="libcext.la"

    cpl_cext_includes=""
    cpl_cext_libraries=""

    AC_ARG_WITH(cext,
                AC_HELP_STRING([--with-cext],
                               [location where Cext is installed]),
                [
                    cpl_with_cext_includes=$withval/include
                    cpl_with_cext_libs=$withval/lib
                ])

    AC_ARG_WITH(cext-includes,
                AC_HELP_STRING([--with-cext-includes],
                               [location of the Cext header files]),
                cpl_with_cext_includes=$withval)

    AC_ARG_WITH(cext-libs,
                AC_HELP_STRING([--with-cext-libs],
                               [location of the Cext library]),
                cpl_with_cext_libs=$withval)

    AC_ARG_ENABLE(cext-test,
                  AC_HELP_STRING([--disable-cext-test],
                                 [disables checks for the Cext library and headers]),
                  cpl_enable_cext_test=$enableval,
                  cpl_enable_cext_test=yes)


    if test "x$cpl_enable_cext_test" = xyes; then

        # Check for the Cext includes

        if test -z "$cpl_with_cext_includes"; then
            cpl_cext_incdirs="/usr/local/include /usr/include"

            test -n "$CPLDIR" && cpl_cext_incdirs="$CPLDIR/include \
                                                   $cpl_cext_incdirs"
        else
            cpl_cext_incdirs="$cpl_with_cext_includes"
        fi

        ESO_FIND_FILE($cpl_cext_check_header, $cpl_cext_incdirs,
                      cpl_cext_includes)


        # Check for the Cext library

        if test -z "$cpl_with_cext_libs"; then
            cpl_cext_libdirs="/usr/local/lib /usr/lib"

            test -n "$CPLDIR" && cpl_cext_libdirs="$CPLDIR/lib \
                                                   $cpl_cext_libdirs"
        else
            cpl_cext_libdirs="$cpl_with_cext_libs"
        fi

        ESO_FIND_FILE($cpl_cext_check_lib, $cpl_cext_libdirs,
                      cpl_cext_libraries)


        if test x"$cpl_cext_includes" = xno || \
            test x"$cpl_cext_libraries" = xno; then
            cpl_cext_notfound=""

            if test x"$cpl_cext_includes" = xno; then
                if test x"$cpl_cext_libraries" = xno; then
                    cpl_cext_notfound="(headers and libraries)"
                else            
                    cpl_cext_notfound="(headers)"
                fi
            else
                cpl_cext_notfound="(libraries)"
            fi

            AC_MSG_ERROR([Cext $cpl_cext_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_cext_libraries, headers $cpl_cext_includes])
        fi


        # Set up the symbols

        CX_INCLUDES="-I$cpl_cext_includes"
        CX_LDFLAGS="-L$cpl_cext_libraries"
        LIBCEXT="-lcext"
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([Cext checks have been disabled! This package may not build!])
        CX_INCLUDES=""
        CX_LDFLAGS=""
        LIBCEXT=""
    fi

    AC_SUBST(CX_INCLUDES)
    AC_SUBST(CX_LDFLAGS)
    AC_SUBST(LIBCEXT)

])


#
# CPL_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the CPL and C extension libraries. If an
# argument is provided the symbols are setup for building CPL, if no
# argument is given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([CPL_CREATE_SYMBOLS],
[

    if test -z "$1"; then
        LIBCPLCORE='-lcplcore'
        LIBCPLDFS='-lcpldfs'
        LIBCPLDRS='-lcpldrs'
        LIBCPLUI='-lcplui'
#		LIBRECIPES='-lrecipes'
        # Third party libraries
        LIBCEXT='-lcext'
    else
        LIBCPLCORE='$(top_builddir)/cplcore/libcplcore.la'
        LIBCPLDFS='$(top_builddir)/cpldrs/libcpldfs.la'
        LIBCPLDRS='$(top_builddir)/cpldrs/libcpldrs.la'
        LIBCPLUI='$(top_builddir)/cplui/libcplui.la'
#		LIBRECIPES='$(top_builddir)/recipes/librecipes.la'
        # Third party libraries
        LIBCEXT='$(top_builddir)/libcext/cext/libcext.la'
    fi

   AC_SUBST(LIBCPLCORE)
   AC_SUBST(LIBCPLDFS)
   AC_SUBST(LIBCPLDRS)
   AC_SUBST(LIBCPLUI)
#   AC_SUBST(LIBRECIPES)
   AC_SUBST(LIBCEXT)

])


# CPL_CHECK_LIBS
#---------------
# Checks for the CPL libraries and header files.
AC_DEFUN([CPL_CHECK_LIBS],
[

    AC_MSG_CHECKING([for CPL])

    cpl_check_cpl_header="cpl_macros.h"
    cpl_check_cpl_lib="libcplcore.la"

    cpl_includes=""
    cpl_libraries=""

    AC_ARG_WITH(cpl,
                AC_HELP_STRING([--with-cpl],
                               [location where CPL is installed]),
                [
                    cpl_with_cpl_includes=$withval/include
                    cpl_with_cpl_libs=$withval/lib
                ])

    AC_ARG_WITH(cpl-includes,
                AC_HELP_STRING([--with-cpl-includes],
                               [location of the CPL header files]),
                cpl_with_cpl_includes=$withval)

    AC_ARG_WITH(cpl-libs,
                AC_HELP_STRING([--with-cpl-libs],
                               [location of the CPL library]),
                cpl_with_cpl_libs=$withval)

    AC_ARG_ENABLE(cpl-test,
                  AC_HELP_STRING([--disable-cpl-test],
                                 [disables checks for the CPL library and headers]),
                  cpl_enable_cpl_test=$enableval,
                  cpl_enable_cpl_test=yes)


    if test "x$cpl_enable_cpl_test" = xyes; then

        # Check for the CPL includes

        if test -z "$cpl_with_cpl_includes"; then
            cpl_incdirs="/opt/cpl/include \
                         /usr/local/include \
                         /usr/local/cpl/include \
                         /usr/local/include/cpl \
                         /usr/include/cpl \
                         /usr/include"

            test -n "$CPLDIR" && cpl_incdirs="$CPLDIR/include $cpl_incdirs"
        else
            cpl_incdirs="$cpl_with_cpl_includes"
        fi

        ESO_FIND_FILE($cpl_check_cpl_header, $cpl_incdirs, cpl_includes)


        # Check for the CPL libraries

        if test -z "$cpl_with_cpl_libs"; then
            cpl_libdirs="/opt/cpl/lib \
                         /usr/local/lib \
                         /usr/local/cpl/lib \
                         /usr/lib"

            test -n "$CPLDIR" && cpl_libdirs="$CPLDIR/lib $cpl_libdirs"
        else
            cpl_libdirs="$cpl_with_cpl_libs"
        fi

        ESO_FIND_FILE($cpl_check_cpl_lib, $cpl_libdirs, cpl_libraries)


        if test x"$cpl_includes" = xno || test x"$cpl_libraries" = xno; then
            cpl_notfound=""

            if test x"$cpl_includes" = xno; then
                if test x"$cpl_libraries" = xno; then
                    cpl_notfound="(headers and libraries)"
                else            
                    cpl_notfound="(headers)"
                fi
            else
                cpl_notfound="(libraries)"
            fi

            AC_MSG_ERROR([CPL $cpl_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_libraries, headers $cpl_includes])
        fi

        # Set up the symbols

        CPL_INCLUDES="-I$cpl_includes"
        CPL_LDFLAGS="-L$cpl_libraries"

        CPL_CREATE_SYMBOLS
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([CPL checks have been disabled! This package may not build!])
        CPL_INCLUDES=""
        CPL_LDFLAGS=""
        LIBCPLCORE=""
        LIBCPLDFS=""
        LIBCPLDRS=""
        LIBCPLUI=""
    fi

    AC_SUBST(CPL_INCLUDES)
    AC_SUBST(CPL_LDFLAGS)
    AC_SUBST(LIBCPLCORE)
    AC_SUBST(LIBCPLDFS)
    AC_SUBST(LIBCPLDRS)
    AC_SUBST(LIBCPLUI)

])
