/* $Id: sinfo_utl_eff.c,v 1.7 2010/02/18 19:14:41 amodigli Exp $
 *
 * This file is part of the SINFONI Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: amodigli $
 * $Date: 2010/02/18 19:14:41 $
 * $Revision: 1.7 $
 * $Log: sinfo_utl_eff.c,v $
 * Revision 1.7  2010/02/18 19:14:41  amodigli
 * some bug fixes
 *
 * Revision 1.6  2010/02/08 07:14:34  amodigli
 * changed sinfo_utl_efficiency API
 *
 * Revision 1.5  2009/12/15 15:07:32  kmirny
 * efficiency update
 *
 * Revision 1.4  2009/07/27 12:37:43  amodigli
 * removed parlist from sinfo_utl_efficiency() API as not used
 *
 * Revision 1.3  2009/07/14 14:45:03  kmirny
 * new recipe parameters
 *
 * Revision 1.2  2009/06/12 14:20:20  kmirny
 * updating SINFONI efficiency calculation
 *
 * Revision 1.1  2009/06/10 14:57:14  kmirny
 * sinfoni efficiency utility recipe
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 ----------------------------------------------------------------------------*/
#include <string.h>

/* cpl */
#include <cpl.h>

/* irplib */
#include <irplib_utils.h>

#include <sinfo_tpl_utils.h>
#include <sinfo_pfits.h>
#include <sinfo_tpl_dfs.h>
#include <sinfo_key_names.h>
#include <sinfo_pro_types.h>
#include <sinfo_ref_types.h>
#include <sinfo_functions.h>
#include <sinfo_msg.h>
#include <sinfo_error.h>
#include <sinfo_utils_wrappers.h>
#include <sinfo_utl_efficiency.h>

/*-----------------------------------------------------------------------------
                            Functions prototypes
 ----------------------------------------------------------------------------*/

static int sinfo_utl_eff_create(cpl_plugin *) ;
static int sinfo_utl_eff_exec(cpl_plugin *) ;
static int sinfo_utl_eff_destroy(cpl_plugin *) ;
static int sinfo_utl_eff(cpl_parameterlist *, cpl_frameset *) ;

/*-----------------------------------------------------------------------------
                            Static variables
 ----------------------------------------------------------------------------*/

static char sinfo_utl_eff_description[] =
"This recipe calculate a efficiency\n"
"esorex --params sinfo_utl_eff\n"
"esorex --help sinfo_utl_eff\n"
"\n";



/*-----------------------------------------------------------------------------
                                Functions code
 ----------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/**
 * @defgroup sinfo_utl_eff  Recipe to do operations on an image
 */
/*---------------------------------------------------------------------------*/

/**@{*/
/*---------------------------------------------------------------------------*/
/**
  @brief    Build the list of available plugins, for this module.
  @param    list    the plugin list
  @return   0 if everything is ok

  This function is exported.
 */
/*---------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe  *   recipe = cpl_calloc(1, sizeof *recipe ) ;
    cpl_plugin  *   plugin = &recipe->interface ;

    cpl_plugin_init(plugin,
                    CPL_PLUGIN_API,
                    SINFONI_BINARY_VERSION,
                    CPL_PLUGIN_TYPE_RECIPE,
                    "sinfo_utl_eff",
                    "Produce a table with efficiency",
                    sinfo_utl_eff_description,
                    "Konstantin Mirny",
                    "kmirny@eso.org",
                    sinfo_get_license(),
                    sinfo_utl_eff_create,
                    sinfo_utl_eff_exec,
                    sinfo_utl_eff_destroy) ;

    cpl_pluginlist_append(list, plugin) ;

    return 0;
}

/*---------------------------------------------------------------------------*/
/**
  @brief    Setup the recipe options
  @param    plugin  the plugin
  @return   0 if everything is ok

  Create the recipe instance and make it available to the application using
  the interface.
 */
/*---------------------------------------------------------------------------*/
static int sinfo_utl_eff_create(cpl_plugin * plugin)
{
    cpl_recipe      * recipe ;

    /* Get the recipe out of the plugin */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE)
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;
    cpl_error_reset();
    irplib_reset();

    /* Create the parameters list in the cpl_recipe object */
    recipe->parameters = cpl_parameterlist_new() ;

    /* Fill the parameters list */


        /* Return */
    return 0;
}

/*---------------------------------------------------------------------------*/
/**
  @brief    Execute the plugin instance given by the interface
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*---------------------------------------------------------------------------*/
static int sinfo_utl_eff_exec(cpl_plugin * plugin)
{
    cpl_recipe  *   recipe ;
     int code=0;
     cpl_errorstate initial_errorstate = cpl_errorstate_get();

    /* Get the recipe out of the plugin */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE)
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;
    cpl_error_reset();
    irplib_reset();
    code = sinfo_utl_eff(recipe->parameters, recipe->frames) ;


    if (!cpl_errorstate_is_equal(initial_errorstate)) {
        /* Dump the error history since recipe execution start.
           At this point the recipe cannot recover from the error */
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);
    }

    return code ;
}

/*---------------------------------------------------------------------------*/
/**
  @brief    Destroy what has been created by the 'create' function
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*---------------------------------------------------------------------------*/
static int sinfo_utl_eff_destroy(cpl_plugin * plugin)
{
    cpl_recipe  *   recipe ;

    /* Get the recipe out of the plugin */
    if (cpl_plugin_get_type(plugin) == CPL_PLUGIN_TYPE_RECIPE)
        recipe = (cpl_recipe *)plugin ;
    else return -1 ;

    cpl_parameterlist_delete(recipe->parameters) ;
    return 0 ;
}

/*---------------------------------------------------------------------------*/
/**
  @brief    Get the command line options and execute the data reduction
  @param    parlist     the parameters list
  @param    framelist   the frames list
  @return   0 if everything is ok
 */
/*---------------------------------------------------------------------------*/
static int
sinfo_utl_eff( cpl_parameterlist   *   parlist,
               cpl_frameset        *   framelist)
{

   const char          *   name_o = NULL ;

   cpl_frame*				product_frame = 0;
   cpl_propertylist    *   plist = NULL ;
   int pos = 0;
   int npos = 0;
   cpl_frame* frm_sci=NULL;
   cpl_frame* frm_atm_ext=NULL;
   cpl_frame* frm_std_cat=NULL;
   cpl_table* eff_tbl=NULL;

   sinfo_msg("Welcome to SINFONI Pipeline release %d.%d.%d",
             SINFONI_MAJOR_VERSION,SINFONI_MINOR_VERSION,SINFONI_MICRO_VERSION);
   ck0(sinfo_dfs_set_groups(framelist),"Cannot indentify RAW and CALIB frames");

   /* HOW TO GET THE VALUE OF A FITS KEYWORD */
   check(plist=cpl_propertylist_new(),"Cannot create a Property List");


   /* Now performing the data reduction */
   name_o = "eff_res.fits" ;

   check_nomsg(frm_sci=cpl_frameset_find (framelist, PRO_STD_STAR_SPECTRA));
   check_nomsg(frm_std_cat=cpl_frameset_find (framelist, FLUX_STD_CATALOG));
   check_nomsg(frm_atm_ext=cpl_frameset_find (framelist, EXTCOEFF_TABLE));
   sinfo_msg("frm_sci=%p frm_std=%p frm_atm_ext=%p",frm_sci,frm_std_cat,frm_atm_ext);
   check_nomsg(eff_tbl=sinfo_efficiency_compute(frm_sci,frm_std_cat,frm_atm_ext));
   npos = cpl_frameset_get_size(framelist);
   for (pos = 0; pos < npos; pos++)
   {
      cpl_frame* pframe = cpl_frameset_get_frame(framelist, pos);
      cpl_frame_group group = cpl_frame_get_group(pframe);
      if (CPL_FRAME_GROUP_PRODUCT == group)
      {
         check_nomsg(cpl_frame_set_group (pframe, CPL_FRAME_GROUP_CALIB));
      }
   }
   /* Create product frame */
   check_nomsg(product_frame = cpl_frame_new());
   check_nomsg(cpl_frame_set_filename(product_frame, name_o)) ;
   check_nomsg(cpl_frame_set_tag(product_frame,"sinfo_efficiency" )) ;
   check_nomsg(cpl_frame_set_type(product_frame, CPL_FRAME_TYPE_TABLE)) ;
   check_nomsg(cpl_frame_set_group(product_frame, CPL_FRAME_GROUP_PRODUCT)) ;
   check(cpl_frame_set_level(product_frame, CPL_FRAME_LEVEL_FINAL),
         "Error while initialising the product frame") ;

   /* Add DataFlow keywords */
   check_nomsg(cpl_propertylist_erase_regexp(plist, "^ESO PRO CATG",0));
   /* Log the saved file in the input frameset */
   check_nomsg(cpl_frameset_insert(framelist, product_frame)) ;
   check(cpl_dfs_setup_product_header(plist,
                                      product_frame,
                                      framelist,
                                      parlist,
                                      "sinfo_utl_eff",
                                      "SINFONI",
                                      KEY_VALUE_HPRO_DID,NULL),
         "Problem in the product DFS-compliance") ;
   sinfo_free_propertylist(&plist) ;

  cleanup:
   sinfo_free_propertylist(&plist) ;
   return (cpl_error_get_code()) ? -1 : 0;
}
/**@}*/
